@@ -76,12 +76,15 @@
 
 #define HAVE_IO_H
 #define ENABLE_PCSC
+#define HAVE_WINSCARD_H
 #define DEFAULT_PCSC_PROVIDER "winscard.dll"
 
+#define SC_PKCS15_PROFILE_DIRECTORY "C:\\Program Files\\OpenSC\\profiles"
+
 #define PATH_MAX _MAX_PATH
 
 #ifndef PACKAGE_VERSION
-#define PACKAGE_VERSION "0.11.6"
+#define PACKAGE_VERSION "0.11.7"
 #endif
 
 #ifndef PACKAGE_NAME
