@@ -35,26 +35,15 @@
 
 #include "internal-winscard.h"
 
-/* Default timeout value for SCardGetStatusChange
- * Needs to be increased for some broken PC/SC
- * Lite implementations.
- */
-#ifndef SC_CUSTOM_STATUS_TIMEOUT
-#define SC_STATUS_TIMEOUT 0
-#else
-#define SC_STATUS_TIMEOUT SC_CUSTOM_STATUS_TIMEOUT
-#endif
-
 /* Some windows specific kludge */
 #undef SCARD_PROTOCOL_ANY
 #define SCARD_PROTOCOL_ANY (SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1)
 /* Error printing */
-#define PCSC_ERROR(ctx, desc, rv) sc_error(ctx, desc ": %lx\n", rv);
+#define PCSC_ERROR(ctx, desc, rv) sc_error(ctx, desc ": 0x%08lx\n", rv);
 
 /* Utility for handling big endian IOCTL codes. */
 #define dw2i_be(a, x) ((((((a[x] << 8) + a[x+1]) << 8) + a[x+2]) << 8) + a[x+3])
 
-#define GET_SLOT_PTR(s, i) (&(s)->slot[(i)])
 #define GET_PRIV_DATA(r) ((struct pcsc_private_data *) (r)->drv_data)
 #define GET_SLOT_DATA(r) ((struct pcsc_slot_data *) (r)->drv_data)
 
@@ -118,13 +107,12 @@ static int pcsc_ret_to_error(long rv)
 	case SCARD_E_SHARING_VIOLATION:
 		return SC_ERROR_READER;
 #ifdef SCARD_E_NO_READERS_AVAILABLE /* Older pcsc-lite does not have it */
-	case SCARD_E_NO_READERS_AVAILABLE: /* Ideelabor #11 */
-		/* Or SC_ERROR_NO_READERS_FOUND 
-		 * or SC_ERROR_READER ? 
-		 * As it currently happens on runtime after a reader has been found once already
-		 * the more suitable error seems to be detached error */
-		return SC_ERROR_READER_DETACHED;
+	case SCARD_E_NO_READERS_AVAILABLE:
+		return SC_ERROR_NO_READERS_FOUND;
 #endif
+        case SCARD_E_NO_SERVICE:
+                /* If the service is (auto)started, there could be readers later */
+                return SC_ERROR_NO_READERS_FOUND;
 	default:
 		return SC_ERROR_UNKNOWN;
 	}
@@ -287,12 +275,12 @@ static int refresh_slot_attributes(sc_reader_t *reader, sc_slot_info_t *slot)
 		pslot->reader_state.dwCurrentState = pslot->reader_state.dwEventState;
 	}
 
-	ret = priv->gpriv->SCardGetStatusChange(priv->gpriv->pcsc_ctx, SC_STATUS_TIMEOUT, &pslot->reader_state, 1);
+	ret = priv->gpriv->SCardGetStatusChange(priv->gpriv->pcsc_ctx, 0, &pslot->reader_state, 1);
 	if (ret == (LONG)SCARD_E_TIMEOUT) { /* timeout: nothing changed */
 		slot->flags &= ~SCARD_STATE_CHANGED;
 		return 0;
 	}
-	if (ret != 0) {
+	if (ret != SCARD_S_SUCCESS) {
 		PCSC_ERROR(reader->ctx, "SCardGetStatusChange failed", ret);
 		return pcsc_ret_to_error(ret);
 	}
@@ -317,7 +305,7 @@ static int refresh_slot_attributes(sc_reader_t *reader, sc_slot_info_t *slot)
 #else
 		/* On windows, SCARD_STATE_CHANGED is turned on by lots of
 		 * other events, so it gives us a lot of false positives.
-		 * But if it's off, there really no change */
+		 * But if it's off, there really was no change */
 		if (pslot->reader_state.dwEventState & SCARD_STATE_CHANGED) {
 			maybe_changed = 1;
 		}
@@ -328,8 +316,8 @@ static int refresh_slot_attributes(sc_reader_t *reader, sc_slot_info_t *slot)
 		slot->flags &= ~SC_SLOT_CARD_CHANGED;
 		if (maybe_changed) {
 			if (old_flags & SC_SLOT_CARD_PRESENT) {
-				DWORD readers_len = 0, state, prot, atr_len = 32;
-				unsigned char atr[32];
+				DWORD readers_len = 0, state, prot, atr_len = SC_MAX_ATR_SIZE;
+				unsigned char atr[SC_MAX_ATR_SIZE];
 				LONG rv = priv->gpriv->SCardStatus(pslot->pcsc_card, NULL, &readers_len,
 					&state,	&prot, atr, &atr_len);
 				if (rv == (LONG)SCARD_W_REMOVED_CARD)
@@ -405,7 +393,7 @@ static int pcsc_wait_for_event(sc_reader_t **readers,
 	}
 
 	ret = priv->gpriv->SCardGetStatusChange(pcsc_ctx, 0, rgReaderStates, nslots);
-	if (ret != 0) {
+	if (ret != SCARD_S_SUCCESS) {
 		PCSC_ERROR(ctx, "SCardGetStatusChange(1) failed", ret);
 		return pcsc_ret_to_error(ret);
 	}
@@ -461,7 +449,7 @@ static int pcsc_wait_for_event(sc_reader_t **readers,
 				continue;
 			return SC_ERROR_EVENT_TIMEOUT;
 		}
-		if (ret != 0) {
+		if (ret != SCARD_S_SUCCESS) {
 			PCSC_ERROR(ctx, "SCardGetStatusChange(2) failed", ret);
 			return pcsc_ret_to_error(ret);
 		}
@@ -534,7 +522,7 @@ static int pcsc_connect(sc_reader_t *reader, sc_slot_info_t *slot)
 	rv = priv->gpriv->SCardConnect(priv->gpriv->pcsc_ctx, priv->reader_name,
 			  priv->gpriv->connect_exclusive ? SCARD_SHARE_EXCLUSIVE : SCARD_SHARE_SHARED,
 			  SCARD_PROTOCOL_ANY, &card_handle, &active_proto);
-	if (rv != 0) {
+	if (rv != SCARD_S_SUCCESS) {
 		PCSC_ERROR(reader->ctx, "SCardConnect failed", rv);
 		return pcsc_ret_to_error(rv);
 	}
@@ -615,7 +603,7 @@ static int pcsc_connect(sc_reader_t *reader, sc_slot_info_t *slot)
 			} else
 				sc_debug(reader->ctx, "Inconsistent TLV from reader!");
 		} else {
-			sc_debug(reader->ctx, "SCardControl failed %d", rv);
+		        PCSC_ERROR(reader->ctx, "SCardControl failed", rv);
 		}
 	}
 	return SC_SUCCESS;
@@ -694,16 +682,13 @@ static int pcsc_unlock(sc_reader_t *reader, sc_slot_info_t *slot)
 
 static int pcsc_release(sc_reader_t *reader)
 {
-	int i;
 	struct pcsc_private_data *priv = GET_PRIV_DATA(reader);
 
 	free(priv->reader_name);
 	free(priv);
-	for (i = 0; i < reader->slot_count; i++) {
-		if (reader->slot[i].drv_data != NULL) {
-			free(reader->slot[i].drv_data);
-			reader->slot[i].drv_data = NULL;
-		}
+	if (reader->slot[0].drv_data != NULL) {
+		free(reader->slot[0].drv_data);
+		reader->slot[0].drv_data = NULL;
 	}
 	return SC_SUCCESS;
 }
@@ -798,10 +783,17 @@ static int pcsc_init(sc_context_t *ctx, void **reader_data)
 		gpriv->SCardListReaders = (SCardListReaders_t)lt_dlsym(gpriv->dlhandle, "SCardListReadersA");
 	
 	/* If we have SCardGetAttrib it is correct API */
-	if (lt_dlsym(gpriv->dlhandle, "SCardGetAttrib") != NULL)
-		gpriv->SCardControl = (SCardControl_t)lt_dlsym(gpriv->dlhandle, "SCardControl");
-	else
+	if (lt_dlsym(gpriv->dlhandle, "SCardGetAttrib") != NULL) {
+#ifdef __APPLE__
+		gpriv->SCardControl = (SCardControl_t)lt_dlsym(gpriv->dlhandle, "SCardControl132");
+#endif
+		if (gpriv->SCardControl == NULL) {
+	                gpriv->SCardControl = (SCardControl_t)lt_dlsym(gpriv->dlhandle, "SCardControl");
+		}
+	}
+	else {
 		gpriv->SCardControlOLD = (SCardControlOLD_t)lt_dlsym(gpriv->dlhandle, "SCardControl");
+	}
 
 	if (
 		gpriv->SCardReleaseContext == NULL ||
@@ -854,15 +846,19 @@ static int pcsc_detect_readers(sc_context_t *ctx, void *prv_data)
 	struct pcsc_global_private_data *gpriv = (struct pcsc_global_private_data *) prv_data;
 	LONG rv;
 	DWORD reader_buf_size;
-	char *reader_buf = NULL, *p;
+	char *reader_buf = NULL, *reader_name;
 	const char *mszGroups = NULL;
 	int ret = SC_ERROR_INTERNAL;
 
+	SC_FUNC_CALLED(ctx, 3);
+
 	if (!gpriv) {
 		ret = SC_ERROR_NO_READERS_FOUND;
 		goto out;
 	}
 
+	sc_debug(ctx, "Probing pcsc readers");
+
 	do {
 		if (gpriv->pcsc_ctx == -1) {
 			/*
@@ -878,13 +874,17 @@ static int pcsc_detect_readers(sc_context_t *ctx, void *prv_data)
 		}
 		if (rv != SCARD_S_SUCCESS) {
 			if (rv != SCARD_E_INVALID_HANDLE) {
+				PCSC_ERROR(ctx, "SCardListReaders failed", rv);
 				ret = pcsc_ret_to_error(rv);
 				goto out;
 			}
 
+			sc_debug(ctx, "Establish pcsc context");
+
 			rv = gpriv->SCardEstablishContext(SCARD_SCOPE_USER,
 					      NULL, NULL, &gpriv->pcsc_ctx);
 			if (rv != SCARD_S_SUCCESS) {
+				PCSC_ERROR(ctx, "SCardEstablishContext failed", rv);
 				ret = pcsc_ret_to_error(rv);
 				goto out;
 			}
@@ -901,10 +901,11 @@ static int pcsc_detect_readers(sc_context_t *ctx, void *prv_data)
 	rv = gpriv->SCardListReaders(gpriv->pcsc_ctx, mszGroups, reader_buf,
 	                      (LPDWORD) &reader_buf_size);
 	if (rv != SCARD_S_SUCCESS) {
+		PCSC_ERROR(ctx, "SCardListReaders failed", rv);
 		ret = pcsc_ret_to_error(rv);
 		goto out;
 	}
-	for (p = reader_buf; *p != '\x0'; p += strlen (p) + 1) {
+	for (reader_name = reader_buf; *reader_name != '\x0'; reader_name += strlen (reader_name) + 1) {
 		sc_reader_t *reader = NULL;
 		struct pcsc_private_data *priv = NULL;
 		struct pcsc_slot_data *pslot = NULL;
@@ -918,7 +919,7 @@ static int pcsc_detect_readers(sc_context_t *ctx, void *prv_data)
 				ret = SC_ERROR_INTERNAL;
 				goto err1;
 			}
-			if (reader2->ops == &pcsc_ops && !strcmp (reader2->name, p)) {
+			if (reader2->ops == &pcsc_ops && !strcmp (reader2->name, reader_name)) {
 				found = 1;
 			}
 		}
@@ -928,6 +929,8 @@ static int pcsc_detect_readers(sc_context_t *ctx, void *prv_data)
 			continue;
 		}
 
+		sc_debug(ctx, "Found new pcsc reader '%s'", reader_name);
+
 		if ((reader = (sc_reader_t *) calloc(1, sizeof(sc_reader_t))) == NULL) {
 			ret = SC_ERROR_OUT_OF_MEMORY;
 			goto err1;
@@ -945,12 +948,12 @@ static int pcsc_detect_readers(sc_context_t *ctx, void *prv_data)
 		reader->ops = &pcsc_ops;
 		reader->driver = &pcsc_drv;
 		reader->slot_count = 1;
-		if ((reader->name = strdup(p)) == NULL) {
+		if ((reader->name = strdup(reader_name)) == NULL) {
 			ret = SC_ERROR_OUT_OF_MEMORY;
 			goto err1;
 		}
 		priv->gpriv = gpriv;
-		if ((priv->reader_name = strdup(p)) == NULL) {
+		if ((priv->reader_name = strdup(reader_name)) == NULL) {
 			ret = SC_ERROR_OUT_OF_MEMORY;
 			goto err1;
 		}
@@ -990,7 +993,7 @@ out:
 	if (reader_buf != NULL)
 		free (reader_buf);
 
-	return ret;
+	SC_FUNC_RETURN(ctx, 3, ret);
 }
 
 static int
